from flask import Flask
from sqlalchemy import create_engine,String,Integer,Column
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker,scoped_session
from graphene_sqlalchemy import SQLAlchemyObjectType
from flask_graphql import GraphQLView
import graphene

HOST = '127.0.0.1'
PORT = 3306
USER = 'root'
PWD = 'root'
DATABASE = 'acmy'

DB_URI = f'mysql+pymysql://{USER}:{PWD}@{HOST}:{PORT}/{DATABASE}?charset=utf8'

engine = create_engine(DB_URI)

Base = declarative_base(engine)
db_session = scoped_session(sessionmaker(bind = engine))
Base.query = db_session.query_property() # 要使用get_query方法必须先调用这个

class Test(Base):
    __tablename__ =  'test03'
    uname = Column(String(6),nullable=False)
    id = Column(Integer,primary_key=True,autoincrement=True)

# Base.metadata.drop_all()
# Base.metadata.create_all()


class Tests(SQLAlchemyObjectType):
    class Meta:
        model = Test
# 增
class Mutation_data(graphene.Mutation):
    t = graphene.Field(Tests)
    flag = graphene.Boolean()
    class Arguments:
        uname = graphene.String()
        id = graphene.Int()
    def mutate(self,info,uname,id):
        try:
            t1 = Test(uname = uname ,id  = id ) 
            db_session.add(t1)
            db_session.commit()
            return Mutation_data(t = t1,flag = True) 
        except Exception as e:
            print(e)
            return {
                'tip':'添加失败'
            }
# 删
class Mutation_data1(graphene.Mutation):
    t = graphene.Field(Tests)
    flag = graphene.Boolean()
    class Arguments:
        uname = graphene.String()
    def mutate(self,info,uname):
        try:
            if uname == '李喆':
                r = db_session.query(Test).filter(Test.uname == uname)
                db_session.delete(r)
                db_session.commit()
                return Mutation_data(t = r,flag = True) 
        except Exception as e:
            print(e)
            return {
                'tip':'删除失败'
            }  
# 改
class Mutation_data2(graphene.Mutation):
    t = graphene.Field(Tests)
    flag = graphene.Boolean()
    class Arguments:
        uname = graphene.String()
    def mutate(self,info,uname):
        try:
            if uname == '准程序员':
                re = db_session.query(Test).filter(Test.uname == '程序员')
                re.uname = uname
                db_session.commit()
        except Exception as e:
            print(e)
            return {
                'tip':'修改失败'
            } 
class Mutation(graphene.ObjectType):
    mutation_data = Mutation_data.Field()
    mutation_data1 = Mutation_data1.Field()
    mutation_data2 = Mutation_data2.Field()


# 查
class Query(graphene.ObjectType):
    tests = graphene.List(Tests)
    testd = graphene.Field(Tests,id = graphene.ID())

    def resolve_tests(self,info):
        query = Tests.get_query(info) # 解析tests相当于解析TESTs,但是这其中的字段时来自数据库的model，所以getquery方法其实解析的就是通过这个类数据库的model的字段
        return query.all() # 查询所有数据
    def resolve_testd(self,info,id):
        q = Tests.get_node(info,id) # 根据id去查询数据
        return q



if __name__ == '__main__':
    app = Flask(__name__)
    schema = graphene.Schema(query= Query,mutation=Mutation)
    app.add_url_rule('/test/',view_func=GraphQLView.as_view('test',schema = schema,graphiql = True))
    app.run(debug=True)
    




